# ####################################################################
# addnic-iedn.py add an iedn nic to a virtual server
# A successful run will return: 'post status 201 post reason Created
# Updated 9/14/12 to uppercase the devnum and vswitname - MAT
# Updated 2/10/13 to use an array for the networks - MAT
# Updated 2/10/13 to check for 1 network for access mode - MAT
# ####################################################################
import sys, httplib, urllib, json

# argv[0] is the script source
if (len(sys.argv) < 11) or (len(sys.argv) > 27):
   print "usage:"
   print "addnic-iedn.py <ip> <hmc-user> <password> <nodename> <hiper-name> <virtual-server-name> <devno> <vswitch-name> <osa-type> <port-mode> <network1-name> <network2-name> <networkn-name>"
   print "example:  addnic-iedn.py 9.12.16.241 ensadmin password P93 VML1 GSSF17 1700 IEDN27 osd access zBX_205 "
   print "  or  :"
   print "example:  addnic-iedn.py 9.12.16.241 ensadmin password P93 VML1 GSSF17 1700 IEDN27 osx trunk zBX_205 zBX_206 zBX_227"
   print "       "
   print "The script can assign up to 16 networks to an adapter in trunk mode.  An access mode adapter can be assigned to 1 network"
   exit()
# use the command line arguments for the params (note: no further error checking being done)

host = str(sys.argv[1])
port = 6794
ensuser = str(sys.argv[2])
enspw = str(sys.argv[3]) 
nodename = str(sys.argv[4])
hname = str(sys.argv[5])
vname = str(sys.argv[6])
devnum = str(sys.argv[7])
vswitname = str(sys.argv[8])
osatype = str(sys.argv[9])
portmode = str(sys.argv[10])

if (portmode == 'access') and (len(sys.argv) > 12): 
   print >>sys.stderr, "Only 1 network can be specified for an access mode NIC"
   exit()

print >>sys.stderr, "Adding a VNIC to VHOST " + vname + " in Hipervisor: " + hname + " on Node: " + nodename + " on virtual dev " + devnum + " on VSWITCH " + vswitname + " on  specified networks.  OSA type is: " + osatype + " port-mode: " + portmode
print >>sys.stderr, " "
print >>sys.stderr, "A successful run returns: post status 201 post reason Created"

################################
####### Setup Network counters
################################
netnum = (len(sys.argv) - 11)
netspot = 11

# ##########################################################################
# Convert the device number and vswitname to upper 
# ##########################################################################

devnumup = devnum.upper()
vswitnameup = vswitname.upper()

##print >>sys.stderr, "devnumup: ",devnumup

devnum = devnumup
vswitname = vswitnameup

###########################################################################
# Logon
###########################################################################

headers = {"Content-type":"application/json","Accept":"*/*"}
# Login credentials live in body 
body = json.dumps({"userid":ensuser,"password":enspw})

conn = httplib.HTTPSConnection(host,port)

conn.request("POST", "/api/session",body,headers)
response = conn.getresponse()

# this session header is important.
apiSessionHeader = response.getheader( "X-API-Session" );
# print >>sys.stderr, "session header:", apiSessionHeader

###########################################################################
# Query for Ensemble ID
###########################################################################
# Notice that there is a session header provided here so it takes advantage of the previous
# logon code. 
headers = {"Content-type":"application/json","Accept":"*/*","X-API-Session":apiSessionHeader}

conn = httplib.HTTPSConnection(host,port)

conn.request("GET", "/api/ensembles", "", headers)
response = conn.getresponse()

mystuff = json.loads(response.read())

# try to get parameter value from target
if len(mystuff) == 1:
   # then we have a list nested in here somewhere
   # and have to dig deeper to get the list of objects
   for x in mystuff: 
      mylist = mystuff[x]
else:
   # we already got our object; pack it in a list
   # so we can handle both cases in same way
   mylist = [ mystuff ]

   # now go through the list and print the selected item
for y in mylist:
   ensembleid = y.get("object-uri")
   print >>sys.stderr, "Here is the ensemble ID for ", hname
   print >>sys.stderr, "  ", ensembleid

###########################################################################
# Query for NODE ID
###########################################################################
# Notice that there is a session header provided here so it takes advantage of the previous
# logon code. 
headers = {"Content-type":"application/json","Accept":"*/*","X-API-Session":apiSessionHeader}

conn = httplib.HTTPSConnection(host,port)

conn.request("GET", ensembleid + "/nodes", "", headers)
response = conn.getresponse()

mystuff = json.loads(response.read())

# try to get parameter value from target
if len(mystuff) == 1:
#      # then we have a list nested in here somewhere
#      # and have to dig deeper to get the list of objects
   for x in mystuff: 
      mylist = mystuff[x]
else:
      mylist = [ mystuff ]

   # now go through the list and print the selected item
for y in mylist:
   if y.get("name") == nodename:
      nodeid = y.get("element-uri")
      print >>sys.stderr, "Here is the Node ID (element-uri) for ", nodename
      print >>sys.stderr, "  ", nodeid

###########################################################################
# Query for member from NODE properties
###########################################################################
# Notice that there is a session header provided here so it takes advantage of the previous
# logon code. 
headers = {"Content-type":"application/json","Accept":"*/*","X-API-Session":apiSessionHeader}

conn = httplib.HTTPSConnection(host,port)

conn.request("GET", nodeid, "", headers)
response = conn.getresponse()

mystuff = json.loads(response.read())

# try to get parameter value from target
if len(mystuff) == 1:

#      # then we have a list nested in here somewhere
#      # and have to dig deeper to get the list of objects
   for x in mystuff: 
      mylist = mystuff[x]
else:

#      # we already got our object; pack it in a list
#      # so we can handle both cases in same way
   mylist = [ mystuff ]

   # now go through the list and print the selected item
for y in mylist:

    nodemember = y.get("member")
    print >>sys.stderr, "Here is the MemberID (member) for ", nodename
    print >>sys.stderr, "  ", nodemember


###########################################################################
# Query for virtualization-hosts ID
###########################################################################
# Notice that there is a session header provided here so it takes advantage of the previous
# logon code. 
headers = {"Content-type":"application/json","Accept":"*/*","X-API-Session":apiSessionHeader}

conn = httplib.HTTPSConnection(host,port)

conn.request("GET", nodemember + "/virtualization-hosts", "", headers)
response = conn.getresponse()

mystuff = json.loads(response.read())

## try to get parameter value from target
if len(mystuff) == 1:

   # then we have a list nested in here somewhere
   # and have to dig deeper to get the list of objects
   for x in mystuff: 
      mylist = mystuff[x]
else:

   # we already got our object; pack it in a list
   # so we can handle both cases in same way
   mylist = [ mystuff ]
   # now go through the list and print the selected item
for y in mylist:

   if y.get("name") == hname:
      hiperid = y.get("object-uri")
      print >>sys.stderr, "Here is the virtualization-host ID (object-uri) for ", hname
      print >>sys.stderr, "  ", hiperid

   
###########################################################################
# Query for virtual server ID
###########################################################################
# Notice that there is a session header provided here so it takes advantage of the previous
# logon code. 
headers = {"Content-type":"application/json","Accept":"*/*","X-API-Session":apiSessionHeader}

conn = httplib.HTTPSConnection(host,port)

conn.request("GET", hiperid + "/virtual-servers", "", headers)
response = conn.getresponse()

mystuff = json.loads(response.read())

## try to get parameter value from target
if len(mystuff) == 1:

   # then we have a list nested in here somewhere
   # and have to dig deeper to get the list of objects
   for x in mystuff: 
      mylist = mystuff[x]
else:

   # we already got our object; pack it in a list
   # so we can handle both cases in same way
   mylist = [ mystuff ]
   # now go through the list and print the selected item
for y in mylist:

   if y.get("name") == vname:

      vsid = y.get("object-uri")
      print >>sys.stderr, "Here is the virtual-server ID (object-uri) for ", vname
      print >>sys.stderr, "  ", vsid

###########################################################################
# Query for vswitch ID
###########################################################################
# Notice that there is a session header provided here so it takes advantage of the previous
# logon code. 
headers = {"Content-type":"application/json","Accept":"*/*","X-API-Session":apiSessionHeader}

conn = httplib.HTTPSConnection(host,port)

conn.request("GET", hiperid + "/virtual-switches", "", headers)
response = conn.getresponse()
### print >>sys.stderr, "post status", response.status, "post reason", response.reason
mystuff = json.loads(response.read())
### print >>sys.stderr, "body:", mystuff
### print >>sys.stderr

## try to get parameter value from target
if len(mystuff) == 1:
###   print >>sys.stderr, "length of mystuff is 1", mystuff
   # then we have a list nested in here somewhere
   # and have to dig deeper to get the list of objects
   for x in mystuff: 
      mylist = mystuff[x]
else:
   print >>sys.stderr, "length of mystuff is not 1", mystuff
   # we already got our object; pack it in a list
   # so we can handle both cases in same way
   mylist = [ mystuff ]
   # now go through the list and print the selected item
for y in mylist:
#   print >>sys.stderr, "Here is the vswitch name we are looking for: ", vswitchname
   if y.get("name") == vswitname:
#      # print parameter matching "hname"
#      print json.dumps(y.get("name"),sort_keys=True, indent=4)
      vswitchid = y.get("element-uri")
      print >>sys.stderr, "Here is the virtual-switch ID (element-uri) for ", vswitname
      print >>sys.stderr, "  ", vswitchid
#   else:
#      # don't do anything


##########################################################################
#  Network processing
##########################################################################
print >>sys.stderr, "Processing networks.  The number of networks is: "
print >>sys.stderr, netnum
##
## Initialize our list - 
vnetarray = ["1"] * netnum
##
for i in range(0,netnum):
   vnetname = str(sys.argv[netspot])
   print >>sys.stderr, "Processing network " + vnetname
   netspot = netspot + 1


###########################################################################
# Query for virtual network ID
###########################################################################
# Notice that there is a session header provided here so it takes advantage of the previous
# logon code. 

   headers = {"Content-type":"application/json","Accept":"*/*","X-API-Session":apiSessionHeader}

   conn = httplib.HTTPSConnection(host,port)

   conn.request("GET", ensembleid + "/virtual-networks", "", headers)
   response = conn.getresponse()

   mystuff = json.loads(response.read())

   ## try to get parameter value from target
   if len(mystuff) == 1:
   
      # then we have a list nested in here somewhere
      # and have to dig deeper to get the list of objects
      for x in mystuff: 
         mylist = mystuff[x]
   else:
   
      # we already got our object; pack it in a list
      # so we can handle both cases in same way
      mylist = [ mystuff ]
   # now go through the list and print the selected item
   for y in mylist:

      if y.get("name") == vnetname:

         vneturi = y.get("object-uri")
         print >>sys.stderr, "Here is the virtual-network ID (object-uri) for ", vnetname
         print >>sys.stderr, "  ", vneturi

   # Take the vneturi and put it in the vnetarray
   vnetarray[i] = vneturi       

###########################################################################
#### SET POST BODY
###########################################################################

request_body = json.dumps({"virtual-device-address":devnum,"type":osatype,"interface-type":"virtual-iedn","switch-uri":vswitchid,"port-mode":portmode,"network-uris":vnetarray})

print >>sys.stderr, "request_body: ", request_body

# ##########################################################################
# POST TO ADD ADAPTER
# ##########################################################################
# Notice that there is a session header provided here so it takes advantage of the previous
# logon code. 
headers = {"Content-type":"application/json","Accept":"*/*","X-API-Session":apiSessionHeader}

print >>sys.stderr, "\nAttempting POST to add nic:"

conn = httplib.HTTPSConnection(host,port)

conn.request("POST", vsid + "/network-adapters", request_body, headers)

response = conn.getresponse()
print >>sys.stderr, "post status", response.status, "post reason", response.reason
mystuff = json.loads(response.read())
print >>sys.stderr, "body:", mystuff
print >>sys.stderr



# ##########################################################################
# Now we'll logoff
# ##########################################################################
headers = {"Content-type":"application/json","Accept":"*/*","X-API-Session":apiSessionHeader}
print >>sys.stderr, "\nAttempting LOGOFF:"

conn = httplib.HTTPSConnection(host,port)

conn.request("DELETE", "/api/session/this-session","",headers)
response = conn.getresponse()
print >>sys.stderr, "delete status", response.status, "delete reason", response.reason
print >>sys.stderr, "body:", response.read()

