# ####################################################################
# addnic-qdio.py add a qdio nic to a virtual server
# A successful run will return: 'post status 201 post reason Created
# Created 7/24/13 from addnic-iedn.py - MAT
# Hardcode the osa type to osd since osx isn't a choice here
# ####################################################################
import sys, httplib, urllib, json

# argv[0] is the script source
if (len(sys.argv) != 9) and (len(sys.argv) != 11):
   print "usage:"
   print "  addnic-qdio.py <ip> <hmc-user> <password> <nodename> <hiper-name> <virtual-server-name> <devno> <vswitch-name> <port-mode> <VLAN>"
   print "  or: "
   print "  addnic-qdio.py <ip> <hmc-user> <password> <nodename> <hiper-name> <virtual-server-name> <devno> <vswitch-name> <port-mode> <VLAN1-VLAN2>"
   print "  or: "
   print "  addnic-qdio.py <ip> <hmc-user> <password> <nodename> <hiper-name> <virtual-server-name> <devno> <vswitch-name> <port-mode> <\"VLAN1 VLAN2 VLAN3\"> "
   print "  or:    "
   print "  addnic-qdio.py <ip> <hmc-user> <password> <nodename> <hiper-name> <virtual-server-name> <devno> <vswitch-name> "
   print "                                                                                                                             "
   print "  example: addnic-qdio.py 9.12.16.236 ensadmin password P95 VML2 LINUXVS2 800 VSQDIO1 access 1208"
   print "  or: "
   print "  example: addnic-qdio.py 9.12.16.236 ensadmin password P95 VML2 LINUXVS2 800 VSQDIO1 trunk 1208-1209"
   print "  or: "
   print "  example: addnic-qdio.py 9.12.16.236 ensadmin password P95 VML2 LINUXVS2 800 VSQDIO1 trunk \"1208 1210 1215\""
   print "  or: "
   print "  example: addnic-qdio.py 9.12.16.236 ensadmin password P95 VML2 LINUXVS2 800 VSUNAWAR "
   print "                                                                                             "
   print " An access mode adapter can be assigned to 1 VLAN "
   print "                                                                                             "
   print " An adapter on a VLAN UNAWARE VSWITCH cannot specify port-mode or VLAN "
   print "                                                 "
   print " A trunk mode adapter can specify either up to four different VLAN IDs separated by spaces, or two VLAN ID ranges using a dash to separate the first and last VLANs in the range."
   print "                                                "

   exit()
# use the command line arguments for the params (note: no further error checking being done)

host = str(sys.argv[1])
port = 6794
ensuser = str(sys.argv[2])
enspw = str(sys.argv[3]) 
nodename = str(sys.argv[4])
hname = str(sys.argv[5])
vname = str(sys.argv[6])
devnum = str(sys.argv[7])
vswitname = str(sys.argv[8])
osatype = "osd"

if len(sys.argv) == 11:
   portmode = str(sys.argv[9])
   vlans = str(sys.argv[10])
   print >>sys.stderr, "Adding a VNIC to VHOST " + vname + " in Hipervisor: " + hname + " on Node: " + nodename + " on virtual dev " +    devnum + " on VSWITCH " + vswitname + " on  vlan: " + vlans + " port-mode: " + portmode

if len(sys.argv) == 9:
   print >>sys.stderr, "Adding a VNIC to VHOST " + vname + " in Hipervisor: " + hname + " on Node: " + nodename + " on virtual dev " +    devnum + " on VSWITCH " + vswitname

print >>sys.stderr, " "
print >>sys.stderr, "A successful run returns: post status 201 post reason Created"

# ##########################################################################
# Convert the device number and vswitname to upper 
# ##########################################################################

devnumup = devnum.upper()
vswitnameup = vswitname.upper()

##print >>sys.stderr, "devnumup: ",devnumup

devnum = devnumup
vswitname = vswitnameup

###########################################################################
# Logon
###########################################################################

headers = {"Content-type":"application/json","Accept":"*/*"}
# Login credentials live in body 
body = json.dumps({"userid":ensuser,"password":enspw})

conn = httplib.HTTPSConnection(host,port)

conn.request("POST", "/api/session",body,headers)
response = conn.getresponse()

# this session header is important.
apiSessionHeader = response.getheader( "X-API-Session" );
# print >>sys.stderr, "session header:", apiSessionHeader

###########################################################################
# Query for Ensemble ID
###########################################################################
# Notice that there is a session header provided here so it takes advantage of the previous
# logon code. 
headers = {"Content-type":"application/json","Accept":"*/*","X-API-Session":apiSessionHeader}

conn = httplib.HTTPSConnection(host,port)

conn.request("GET", "/api/ensembles", "", headers)
response = conn.getresponse()

mystuff = json.loads(response.read())

# try to get parameter value from target
if len(mystuff) == 1:
   # then we have a list nested in here somewhere
   # and have to dig deeper to get the list of objects
   for x in mystuff: 
      mylist = mystuff[x]
else:
   # we already got our object; pack it in a list
   # so we can handle both cases in same way
   mylist = [ mystuff ]

   # now go through the list and print the selected item
for y in mylist:
   ensembleid = y.get("object-uri")
   print >>sys.stderr, "Here is the ensemble ID for ", hname
   print >>sys.stderr, "  ", ensembleid

###########################################################################
# Query for NODE ID
###########################################################################
# Notice that there is a session header provided here so it takes advantage of the previous
# logon code. 
headers = {"Content-type":"application/json","Accept":"*/*","X-API-Session":apiSessionHeader}

conn = httplib.HTTPSConnection(host,port)

conn.request("GET", ensembleid + "/nodes", "", headers)
response = conn.getresponse()

mystuff = json.loads(response.read())

# try to get parameter value from target
if len(mystuff) == 1:
#      # then we have a list nested in here somewhere
#      # and have to dig deeper to get the list of objects
   for x in mystuff: 
      mylist = mystuff[x]
else:
      mylist = [ mystuff ]

   # now go through the list and print the selected item
for y in mylist:
   if y.get("name") == nodename:
      nodeid = y.get("element-uri")
      print >>sys.stderr, "Here is the Node ID (element-uri) for ", nodename
      print >>sys.stderr, "  ", nodeid

###########################################################################
# Query for member from NODE properties
###########################################################################
# Notice that there is a session header provided here so it takes advantage of the previous
# logon code. 
headers = {"Content-type":"application/json","Accept":"*/*","X-API-Session":apiSessionHeader}

conn = httplib.HTTPSConnection(host,port)

conn.request("GET", nodeid, "", headers)
response = conn.getresponse()

mystuff = json.loads(response.read())

# try to get parameter value from target
if len(mystuff) == 1:

#      # then we have a list nested in here somewhere
#      # and have to dig deeper to get the list of objects
   for x in mystuff: 
      mylist = mystuff[x]
else:

#      # we already got our object; pack it in a list
#      # so we can handle both cases in same way
   mylist = [ mystuff ]

   # now go through the list and print the selected item
for y in mylist:

    nodemember = y.get("member")
    print >>sys.stderr, "Here is the MemberID (member) for ", nodename
    print >>sys.stderr, "  ", nodemember


###########################################################################
# Query for virtualization-hosts ID
###########################################################################
# Notice that there is a session header provided here so it takes advantage of the previous
# logon code. 
headers = {"Content-type":"application/json","Accept":"*/*","X-API-Session":apiSessionHeader}

conn = httplib.HTTPSConnection(host,port)

conn.request("GET", nodemember + "/virtualization-hosts", "", headers)
response = conn.getresponse()

mystuff = json.loads(response.read())

## try to get parameter value from target
if len(mystuff) == 1:

   # then we have a list nested in here somewhere
   # and have to dig deeper to get the list of objects
   for x in mystuff: 
      mylist = mystuff[x]
else:

   # we already got our object; pack it in a list
   # so we can handle both cases in same way
   mylist = [ mystuff ]
   # now go through the list and print the selected item
for y in mylist:

   if y.get("name") == hname:
      hiperid = y.get("object-uri")
      print >>sys.stderr, "Here is the virtualization-host ID (object-uri) for ", hname
      print >>sys.stderr, "  ", hiperid

   
###########################################################################
# Query for virtual server ID
###########################################################################
# Notice that there is a session header provided here so it takes advantage of the previous
# logon code. 
headers = {"Content-type":"application/json","Accept":"*/*","X-API-Session":apiSessionHeader}

conn = httplib.HTTPSConnection(host,port)

conn.request("GET", hiperid + "/virtual-servers", "", headers)
response = conn.getresponse()

mystuff = json.loads(response.read())

## try to get parameter value from target
if len(mystuff) == 1:

   # then we have a list nested in here somewhere
   # and have to dig deeper to get the list of objects
   for x in mystuff: 
      mylist = mystuff[x]
else:

   # we already got our object; pack it in a list
   # so we can handle both cases in same way
   mylist = [ mystuff ]
   # now go through the list and print the selected item
for y in mylist:

   if y.get("name") == vname:

      vsid = y.get("object-uri")
      print >>sys.stderr, "Here is the virtual-server ID (object-uri) for ", vname
      print >>sys.stderr, "  ", vsid

###########################################################################
# Query for vswitch ID
###########################################################################
# Notice that there is a session header provided here so it takes advantage of the previous
# logon code. 
headers = {"Content-type":"application/json","Accept":"*/*","X-API-Session":apiSessionHeader}

conn = httplib.HTTPSConnection(host,port)

conn.request("GET", hiperid + "/virtual-switches", "", headers)
response = conn.getresponse()
### print >>sys.stderr, "post status", response.status, "post reason", response.reason
mystuff = json.loads(response.read())
### print >>sys.stderr, "body:", mystuff
### print >>sys.stderr

## try to get parameter value from target
if len(mystuff) == 1:
###   print >>sys.stderr, "length of mystuff is 1", mystuff
   # then we have a list nested in here somewhere
   # and have to dig deeper to get the list of objects
   for x in mystuff: 
      mylist = mystuff[x]
else:
   print >>sys.stderr, "length of mystuff is not 1", mystuff
   # we already got our object; pack it in a list
   # so we can handle both cases in same way
   mylist = [ mystuff ]
   # now go through the list and print the selected item
for y in mylist:
#   print >>sys.stderr, "Here is the vswitch name we are looking for: ", vswitchname
   if y.get("name") == vswitname:
#      # print parameter matching "hname"
#      print json.dumps(y.get("name"),sort_keys=True, indent=4)
      vswitchid = y.get("element-uri")
      print >>sys.stderr, "Here is the virtual-switch ID (element-uri) for ", vswitname
      print >>sys.stderr, "  ", vswitchid
#   else:
#      # don't do anything


###########################################################################
#### SET POST BODY
###########################################################################

if len(sys.argv) == 11:
   request_body = json.dumps({"virtual-device-address":devnum,"type":osatype,"interface-type":"virtual-qdio","switch-uri":vswitchid,"port-mode":portmode,"vlan-ids":vlans})

if len(sys.argv) == 9:
   request_body = json.dumps({"virtual-device-address":devnum,"type":osatype,"interface-type":"virtual-qdio","switch-uri":vswitchid})


###request_body = json.dumps({"virtual-device-address":devnum,"interface-type":"virtual-qdio","switch-uri":vswitchid,"port-mode":portmode,"vlan-ids":vlans})

print >>sys.stderr, "request_body: ", request_body

# ##########################################################################
# POST TO ADD ADAPTER
# ##########################################################################
# Notice that there is a session header provided here so it takes advantage of the previous
# logon code. 
headers = {"Content-type":"application/json","Accept":"*/*","X-API-Session":apiSessionHeader}

print >>sys.stderr, "\nAttempting POST to add nic:"

conn = httplib.HTTPSConnection(host,port)

conn.request("POST", vsid + "/network-adapters", request_body, headers)

response = conn.getresponse()
print >>sys.stderr, "post status", response.status, "post reason", response.reason
mystuff = json.loads(response.read())
print >>sys.stderr, "body:", mystuff
print >>sys.stderr



# ##########################################################################
# Now we'll logoff
# ##########################################################################
headers = {"Content-type":"application/json","Accept":"*/*","X-API-Session":apiSessionHeader}
##print >>sys.stderr, "\nAttempting LOGOFF:"

conn = httplib.HTTPSConnection(host,port)

conn.request("DELETE", "/api/session/this-session","",headers)
response = conn.getresponse()
##print >>sys.stderr, "delete status", response.status, "delete reason", response.reason
##print >>sys.stderr, "body:", response.read()

