###################################################
# 
# Notes:
# original     M. Tebolt
# 
###################################################
import sys, httplib, urllib, json

# argv[0] is the script source
if len(sys.argv) != 8:
   print "usage:"
   print "delnic.py <ip> <hmc-user> <password> <nodename> <hiper-name> <virtual-server-name> <virutal-device-number> "
   print "example:       delnic.py 9.12.16.236 ensadmin password P95 VML1 GSSP28 1700 "
   exit()
# use the command line arguments for the params (note: no further error checking being done)
else:
   host = str(sys.argv[1])
   port = 6794
   ensuser = str(sys.argv[2])
   enspw = str(sys.argv[3]) 
   nodename = str(sys.argv[4])
   hname = str(sys.argv[5])
   vname = str(sys.argv[6])
   vdev = str(sys.argv[7])

print >>sys.stderr, "Deleting " + vdev + " VNIC on VHOST " + vname + " in Hipervisor: " + hname + " on Node: " + nodename
print >>sys.stderr, "Success returns 204 status code"

print >>sys.stderr, "args:",host,ensuser,nodename,hname,vname,vdev

# ##########################################################################
# Logon
# ##########################################################################

headers = {"Content-type":"application/json","Accept":"*/*"}
# Login credentials live in body 
body = json.dumps({"userid":ensuser,"password":enspw})
##print >>sys.stderr, "body: ", body

conn = httplib.HTTPSConnection(host,port)

conn.request("POST", "/api/session",body,headers)
response = conn.getresponse()

# this session header is important.
apiSessionHeader = response.getheader( "X-API-Session" );
##print >>sys.stderr, "session header:", apiSessionHeader

# ##########################################################################
# Query for Ensemble ID
# ##########################################################################
# Notice that there is a session header provided here so it takes advantage of the previous
# logon code. 
headers = {"Content-type":"application/json","Accept":"*/*","X-API-Session":apiSessionHeader}

conn = httplib.HTTPSConnection(host,port)

conn.request("GET", "/api/ensembles", "", headers)
response = conn.getresponse()

mystuff = json.loads(response.read())

# try to get parameter value from target
if len(mystuff) == 1:
   # then we have a list nested in here somewhere
   # and have to dig deeper to get the list of objects
   for x in mystuff: 
      mylist = mystuff[x]
else:
   # we already got our object; pack it in a list
   # so we can handle both cases in same way
   mylist = [ mystuff ]

   # now go through the list and print the selected item
for y in mylist:
   ensembleid = y.get("object-uri")
   print >>sys.stderr, "Here is the ensemble ID for ", hname
   print >>sys.stderr, "  ", ensembleid

# ##########################################################################
# Query for NODE ID
# ##########################################################################
# Notice that there is a session header provided here so it takes advantage of the previous
# logon code. 
headers = {"Content-type":"application/json","Accept":"*/*","X-API-Session":apiSessionHeader}

conn = httplib.HTTPSConnection(host,port)

conn.request("GET", ensembleid + "/nodes", "", headers)
response = conn.getresponse()

mystuff = json.loads(response.read())

# try to get parameter value from target
if len(mystuff) == 1:
#      # then we have a list nested in here somewhere
#      # and have to dig deeper to get the list of objects
   for x in mystuff: 
      mylist = mystuff[x]
else:
      mylist = [ mystuff ]

   # now go through the list and print the selected item
for y in mylist:
   if y.get("name") == nodename:
      nodeid = y.get("element-uri")
      print >>sys.stderr, "Here is the Node ID (element-uri) for ", nodename
      print >>sys.stderr, "  ", nodeid

# ##########################################################################
# Query for member from NODE properties
# ##########################################################################
# Notice that there is a session header provided here so it takes advantage of the previous
# logon code. 
headers = {"Content-type":"application/json","Accept":"*/*","X-API-Session":apiSessionHeader}

conn = httplib.HTTPSConnection(host,port)

conn.request("GET", nodeid, "", headers)
response = conn.getresponse()

mystuff = json.loads(response.read())

# try to get parameter value from target
if len(mystuff) == 1:

#      # then we have a list nested in here somewhere
#      # and have to dig deeper to get the list of objects
   for x in mystuff: 
      mylist = mystuff[x]
else:

#      # we already got our object; pack it in a list
#      # so we can handle both cases in same way
   mylist = [ mystuff ]

   # now go through the list and print the selected item
for y in mylist:

    nodemember = y.get("member")
    print >>sys.stderr, "Here is the MemberID (member) for ", nodename
    print >>sys.stderr, "  ", nodemember


# ##########################################################################
# Query for virtualization-hosts ID
# ##########################################################################
# Notice that there is a session header provided here so it takes advantage of the previous
# logon code. 
headers = {"Content-type":"application/json","Accept":"*/*","X-API-Session":apiSessionHeader}

conn = httplib.HTTPSConnection(host,port)

conn.request("GET", nodemember + "/virtualization-hosts", "", headers)
response = conn.getresponse()

mystuff = json.loads(response.read())

## try to get parameter value from target
if len(mystuff) == 1:

   # then we have a list nested in here somewhere
   # and have to dig deeper to get the list of objects
   for x in mystuff: 
      mylist = mystuff[x]
else:

   # we already got our object; pack it in a list
   # so we can handle both cases in same way
   mylist = [ mystuff ]
   # now go through the list and print the selected item
for y in mylist:

   if y.get("name") == hname:
      hiperid = y.get("object-uri")
      print >>sys.stderr, "Here is the virtualization-host ID (object-uri) for ", hname
      print >>sys.stderr, "  ", hiperid

   
# ##########################################################################
# Query for virtual server ID
# ##########################################################################
# Notice that there is a session header provided here so it takes advantage of the previous
# logon code. 
headers = {"Content-type":"application/json","Accept":"*/*","X-API-Session":apiSessionHeader}

conn = httplib.HTTPSConnection(host,port)

conn.request("GET", hiperid + "/virtual-servers", "", headers)
response = conn.getresponse()

mystuff = json.loads(response.read())

## try to get parameter value from target
if len(mystuff) == 1:

   # then we have a list nested in here somewhere
   # and have to dig deeper to get the list of objects
   for x in mystuff: 
      mylist = mystuff[x]
else:

   # we already got our object; pack it in a list
   # so we can handle both cases in same way
   mylist = [ mystuff ]
   # now go through the list and print the selected item
for y in mylist:

   if y.get("name") == vname:

      vsid = y.get("object-uri")
      print >>sys.stderr, "Here is the virtual-server ID (object-uri) for ", vname
      print >>sys.stderr, "  ", vsid

# ##########################################################################
# DELETE TO REMOVE ADAPTER
# ##########################################################################
# Notice that there is a session header provided here so it takes advantage of the previous
# logon code. 
headers = {"Content-type":"application/json","Accept":"*/*","X-API-Session":apiSessionHeader}

print >>sys.stderr, "\nAttempting DELETE of a Network Adapter:"

conn = httplib.HTTPSConnection(host,port)

conn.request("DELETE", vsid + "/network-adapters/" + vdev, "", headers)
response = conn.getresponse()
print >>sys.stderr, "post status", response.status, "post reason", response.reason

#### Uncomment the following lines for debug
##mystuff = json.loads(response.read())
##print >>sys.stderr, "body:", mystuff
##print >>sys.stderr



# ##########################################################################
# Now we'll logoff
# ##########################################################################
headers = {"Content-type":"application/json","Accept":"*/*","X-API-Session":apiSessionHeader}
print >>sys.stderr, "\nAttempting LOGOFF:"

conn = httplib.HTTPSConnection(host,port)

conn.request("DELETE", "/api/session/this-session","",headers)
response = conn.getresponse()
print >>sys.stderr, "delete status", response.status, "delete reason", response.reason
print >>sys.stderr, "body:", response.read()

